#!/bin/bash

#==================================================================
# Filename: data_storage_ratio.sh
#==================================================================
# Purpose: Prints the data storage ratio for the TPC-H setup.
#==================================================================

if [ -z "$1" -o -z "$2" -o -z "$3" -o -z "$4" ]
then
	echo "Usage:  $0 <scale_factor> <num_disks> <disk_size(GB)> <data_storage_format>" 
	echo " "
	echo " <total_data_storage> -- total data storage used for the test database"
	echo " "
	echo " For the reporting of configured disk capacity, gigabyte (GB) is "
	echo " defined to be 2^30 bytes. Since disk manufacturers typically report"
	echo " disk size using base ten (i.e., GB = 10^9), it may be necessary to"
	echo " convert the advertised size from base ten to base two."
	echo " "
	echo " <data_storage_format> = 1, "
	echo " 			If the disk manufacturers report disk size"
	echo "			base ten (i.e., GB=10^9) instead of 2^30bytes. "
	echo " <data_storage_format> = 0, otherwise.  " 
	echo " "
	exit
fi

sf=$1
nd=$2 
ds=$3
data_storage_format=$4

ts=$(echo $nd*$ds|bc) # total disk storage used for the test database.

if [ $data_storage_format -eq 1 ]
then
	ratio=$(echo "scale=9;($ts*1000000000)/(${sf}*1024*3)"|bc)
else
	ratio=$(echo "scale=9;($ts)/($sf)"|bc)
fi

ratio=$(echo "scale=2;($ratio+0.005)/1"|bc) 
echo $ratio
