#!/bin/bash

#=========================================================================
# Filename: geo_mean_power_test.sh
#=========================================================================
#
# Purpose:
# Compute the geometric mean of the power test.
# This metric will be reported in the FDR (executive summary).
#
# Algorithm:
#
# 1) Then we calculate 
#    	Log(RespTime(Q1)) + Log(RespTime(Q2)) + ... + Log(RespTime(Q22))
#		+ Log(RespTime(RF1)) + Log(RespTime(RF2))
#       where RespTime(a) = Response/Elapsed Time of
#			 query/refresh function a.
#
# 2) Finally we compute the geometric mean of all the response times
#       (10**(sum Log's query and refresh stream response_times)/24)
#
#=========================================================================


if [ -z "$1" ] 
then
  echo "usage:  $0  WITH_RF"
  echo ""
  echo "           - returns geometric mean of Q1-Q22 from stream0.out"
  echo "  		and RF1 and RF2 from update_power.out"
  exit
fi

WITH_RF=$1

# Compute the sum Log's query response times
rows=0
logquerysum=0
logrefreshsum=0
#while calculating the query times for geometric mean, query time should be from start of current query to start of next query time.

calculate_query_elapsed_time.sh 0

for i in `grep "Elapsed time" $TEMPFILES/stream0_elapsed_times.out|cut -d ' ' -f5`
do
	#  echo $i   [debug  tpch_power.sh]
	j=`round_to_tenths.pl $i`

	# echo $j   [debug  rounded value]

	temp=$(echo "l($j)"|bc -l)
	#echo $temp for $j [debug ln x]

	if [ $rows = 0 ]  
	then
		logquerysum=$temp
	else
		logquerysum=$(echo "($temp+$logquerysum)/1"|bc -l)
	fi
	#echo $logquerysum
	rows=`expr $rows + 1`
done

#echo logquerysum=$logquerysum # [debug geo_mean_power_test.sh]

# Compute the sum Log's refresh stream response times
if [ $WITH_RF -ne 0 ]
then
	total_query_refresh=24
	rows=0
	for i in `grep "Elapsed time" $RESULTS/update_power.out | awk '{print $(NF-1)}'`
	do
		# echo $i  [debug  tpch_power.sh]
		j=`round_to_tenths.pl $i`

		#echo $j   [debug  rounded value]

		temp=$(echo "l($j)"|bc -l)
		#	echo $temp for $j [debug ln x]

		if [ $rows = 0 ]  
		then
			logrefreshsum=$temp
		else
			logrefreshsum=$(echo "($temp+$logrefreshsum)/1"|bc -l)
		fi
		
		rows=`expr $rows + 1`
	done
else
	total_query_refresh=22
fi
# echo logrefreshsum=$logrefreshsum  [debug geo_mean_power_test.sh]

sume=$(echo "$logquerysum+$logrefreshsum"|bc -l)
#echo sume=$sume [debug sume logs query and refresh stream ]

sum10=$(echo "$sume/l(10)"|bc -l)
#evaluate the geometric mean 
line="10**($sum10/$total_query_refresh)"
geomean=`compute_expr.pl "$line"`

echo $geomean

