#!/bin/bash

#====================================================================
# Filename: query_sequence.sh
#====================================================================
# Purpose: 
# 	To print the query sequence of the streams runs for 
# 	Power and throughput test.
#====================================================================

if [ -z "$1" -o -z "$2" ]
then
	echo "Usage $0 <SCOPE> <STREAM_COUNT>"
	echo ""
	echo " 		For SCOPE=power, STREAM_COUNT has no relevance "
	exit
fi

scope=$1

STREAM_COUNT=$2
if [ $scope = power ]
then
	end_stream=0
else
	end_stream=$STREAM_COUNT
fi

rm -f $TEMPFILES/sequence.out > /dev/null 2>&1
touch $TEMPFILES/sequence.out

if [ $scope = throughput ]
then
	start_stream=1
else
	start_stream=0
fi

for i in `seq $start_stream $end_stream`
do
	grep 'Elapsed time,q' $RESULTS/stream${i}.out | grep -v ^[0-9]*">" | grep -v datediff \
		| grep -v 'Stream' | cut -d ',' -f 2 | tr -d 'q' \
		| sed -e 's/$/ /' |tr -d '\n' >> $TEMPFILES/sequence.out 
	echo >> $TEMPFILES/sequence.out
done

line=0
for i in `seq 1 22`
do
	if [ $i -eq 1 ]
	then
		line=$(echo $i)
	else
		line=$(echo $line","$i)
	fi
done

echo
echo "Query Sequence in the respective query streams"
echo
echo $line | awk -F"," '{ printf "Query#->"; for (i=1;i<=22;i++) { printf "\t"$i } }' | expand -4

echo
echo 
if [ $scope = power -o $scope = all -o $scope = perf ]
then
	echo "Power Test"
	cat $TEMPFILES/sequence.out | head -1 | awk '{ printf "Stream0->"; for (i=1;i<=22;i++) { printf "\t"$i }}' | expand -4
	start_stream=1
fi

echo
echo
if [ $scope = throughput -o $scope = all -o $scope = perf ]
then
	echo "Throughput Test"
	if [ $scope = throughput ]
	then
		j=0
	else
		j=1
	fi

	for i in `seq $start_stream $end_stream`
	do
		j=`expr $j + 1`
		echo Stream${i}"->" | tr -d '\n'
		cat $TEMPFILES/sequence.out | head -${j} | tail -1 | awk '{ for (k=1;k<=22;k++) { printf "\t"$k }}' | expand -4	
		echo
	done
fi
echo
echo
