#!/bin/bash

if [ -z "$2" ]
then
        echo " Usage: $0 <SCOPE><stream_count/stream_number>"
        echo ""
        echo " Usage Example 1:    $0 all 2"
	echo " Usage Example 2:	  $0 streamwise 4 generates the report for 4th stream."
        echo ""
        exit 1
fi

scope=$1
STREAM_COUNT=$2
if [ $scope = all -o $scope = perf -o $scope = throughput -o $scope = power ]
then
	echo '			Streamwise Query Report'
	echo '			----------------------------'
	echo
fi

if [ $scope = all -o $scope = perf ]
then
	start_stream=0
	end_stream=$STREAM_COUNT
fi

if [ $scope = throughput ]
then
	start_stream=1
	end_stream=$STREAM_COUNT
fi
 
if [ $scope = power ]
then
	start_stream=0
	end_stream=0
fi 

if [ $scope = inter -o $scope = timing ]
then
	STREAM_NUMBER=$2
	start_stream=$STREAM_NUMBER
	end_stream=$STREAM_NUMBER
	nCols=4
fi


for i in `seq $start_stream $end_stream`
do
	calculate_query_elapsed_time.sh $i
	#grep for query execution time from stream${i}.out
	grep 'Elapsed ' $TEMPFILES/stream${i}_elapsed_times.out |cut -d ' ' -f2,5 \
		|sort -t, +0 -1 -n  >$TEMPFILES/querytimes_stream${i}.out	
	
	grep "Stream $i Query" $RESULTS/stream${i}.out | grep -v ^[0-9]*">" | grep START \
		| grep -v dateformat| cut -d ' ' -f 5,8-|tr -s ' '|sort -t, +0 -1 -n  \
			>$TEMPFILES/queryStartTimes_stream${i}.out

	grep "Stream $i Query" $RESULTS/stream${i}.out |grep -v ^[0-9]*">" | grep STOP \
		| grep -v dateformat| cut -d ' ' -f 5,8- | tr -s ' ' | sort -t, +0 -1 -n  \
			>$TEMPFILES/queryStopTimes_stream${i}.out

	grep 'Elapsed' $TEMPFILES/stream${i}_elapsed_times.out|cut -d' ' -f2 > $TEMPFILES/querysequence
	
	os=$(uname)

	if [ $os = AIX ]
	then
		cat -n $TEMPFILES/querysequence |sort +1 -n |tr -s ' '>$TEMPFILES/sorted_querysequence
	else
		cat -n $TEMPFILES/querysequence |sort -k 2 -n |tr -s ' '>$TEMPFILES/sorted_querysequence
	fi

	sortedquerysequence=`cat $TEMPFILES/sorted_querysequence |cut -d '	' -f1|sed -e 's/$/ /' |tr -d '\n'`
		
	echo "Stream$i"
	echo "---------------------------------"
	
	seed=`grep "stream$i " ${STREAMS}/stream_seeds|cut -d ' ' -f2`

	starttime=`grep -i 'Stream START' $RESULTS/stream${i}.out | grep -v ^[0-9]*">" \
		| grep -v dateformat|tr -s ' '|cut -d ' ' -f 6,7`
	endtime=`grep -i 'Stream STOP' $RESULTS/stream${i}.out | grep -v ^[0-9]*">" \
		| grep -v dateformat|tr -s ' '|cut -d ' ' -f 6,7`
	streamelapsed_time=`grep Elapsed $RESULTS/stream${i}.out |grep -v q|grep -v date|cut -d ',' -f2|tr -s ' '`
	echo "Seed used for current run :	$seed"
	echo
	if [ ! $scope = inter ]
	then
		echo "Stream start time stamp	:	$starttime"
		echo
		echo "Stream stop time stamp	:	$endtime"
		echo
	fi
	
	if [ ! $scope = timing ]
	then
		echo "Stream Elapsed time	:	$streamelapsed_time"
		echo
	fi

	if [ ! $scope = inter -a ! $scope = timing ]
	then
 		echo "Query,Sequence,StartTime,		EndTime,		ElapsedTime,AdjustedElapsedTime" \
			|awk -F"," '{ for(k=1;k<=6;k++) { printf $k"\t"; } printf "\n" }'
		
		echo "-----,-------,-----------------------,	-----------------------,	------------,-------------------"|awk -F"," '{ for(k=1;k<=6;k++) { printf $k"\t"; } printf "\n" }'

	elif [ $scope = inter ]
	then
 		echo "Query,Sequence,ElapsedTime,AdjustedElapsedTime" \
			|awk -F"," '{ for(k=1;k<='$nCols';k++) { printf $k"\t"; } printf "\n" }'
		
		echo "-----,-------,------------,-------------------" \
			|awk -F"," '{ for(k=1;k<='$nCols';k++) { printf $k"\t"; } printf "\n" }'
	elif [ $scope = timing ]
	then
 		echo "Query,Sequence,StartTime,		EndTime" \
			|awk -F"," '{ for(k=1;k<='$nCols';k++) { printf $k"\t"; } printf "\n" }'
		
		echo "-----,-------,-----------------------,	-----------------------"\
			|awk -F"," '{ for(k=1;k<='$nCols';k++) { printf $k"\t"; } printf "\n" }'
	
	fi	

	j=1

        if [ ! $scope = timing ]
	then
		totalquerytime=0
		MINQ=9999999999999
		MAXQ=0
		MINQI=0
		MAXQI=0
		logquerysum=0
	fi

	while [ $j -le 22 ]
	do
		query=`head -${j} $TEMPFILES/queryStartTimes_stream${i}.out \
				| tail -1|cut -d ',' -f 1`
		sequence=`echo $sortedquerysequence|cut -d ' ' -f${j}`
		if [ ! $scope = inter ]
		then
			starttime=`head -${j} $TEMPFILES/queryStartTimes_stream${i}.out \
					| tail -1|cut -d ',' -f 2`
			endtime=`head -${j} $TEMPFILES/queryStopTimes_stream${i}.out \
					| tail -1|cut -d ',' -f 2`
		fi

		if [ ! $scope = timing ]
		then
			querytime=`head -${j} $TEMPFILES/querytimes_stream${i}.out \
					| tail -1|cut -d ' ' -f 2`
			adjustedElapsedtime=`round_to_tenths.pl $querytime`
			add_zero_to_query_time=`echo $querytime|grep "^\."`
			if [ ! -z "$add_zero_to_query_time" ]
			then
				querytime=`echo "0$querytime"`
			fi
		fi

		if [ ! $scope = inter -a ! $scope = timing ]
		then
      			echo "$query,$sequence,$starttime,$endtime,$querytime,	$adjustedElapsedtime" \
		 	| sed -e 's/;/:/g' |awk -F"," '{ for(k=1;k<=6;k++) { printf $k"\t"; } printf "\n" }'
		elif [ $scope = inter ]
		then
			echo "$query,$sequence,$querytime,  $adjustedElapsedtime" \
			| sed -e 's/;/:/g' |awk -F"," '{ for(k=1;k<=4;k++) { printf $k"\t"; } printf "\n" }'

		elif [ $scope = timing ]
		then
      			echo "$query,$sequence,$starttime,$endtime" \
		 	| sed -e 's/;/:/g' |awk -F"," '{ for(k=1;k<=4;k++) { printf $k"\t"; } printf "\n" }'
		fi	
		
		if [ ! $scope = timing ]
		then
			totalquerytime=$(echo "scale=3;$totalquerytime + $querytime"|bc)
			logquerytime=$(echo "l($querytime)"|bc -l)
			logquerysum=$(echo "($logquerytime+$logquerysum)/1"|bc -l)

			querytime_int=$(echo `echo "scale=0;$querytime*1000"|bc`|cut -d'.' -f1)
			MINQ_int=$(echo `echo "scale=0;$MINQ*1000"|bc`|cut -d'.' -f1)
			MAXQ_int=$(echo `echo "scale=0;$MAXQ*1000"|bc`|cut -d'.' -f1)

			if [ $querytime_int -lt $MINQ_int ]
			then
				MINQ=$querytime
				MINQI=$j
			fi

			if [ $querytime_int -gt $MAXQ_int ]
			then
				MAXQ=$querytime
				MAXQI=$j
			fi
		fi
		
		j=$(echo $j + 1|bc)
	done 

	if [ ! $scope = timing ]
	then	
		avg=$(echo "scale=3;$totalquerytime/22"|bc)
		avg_rounded=$(round_to_tenths.pl $avg)
		MINQ_rounded=$(round_to_tenths.pl $MINQ)
		MAXQ_rounded=$(round_to_tenths.pl $MAXQ)
		sum10=$(echo "$logquerysum/l(10)"|bc -l)
		line="10**($sum10/22)"
		geomean=`compute_expr.pl "$line"`
		geomean_rounded=$(round_to_tenths.pl $geomean)

		echo
		echo
		echo "Max Qry                     :	Q$MAXQI		$MAXQ_rounded"
		echo
		echo "Min Qry                     :	Q$MINQI		$MINQ_rounded"  
		echo
		echo "Arithmatic mean of queries  :	$avg_rounded"
		echo
		echo "Geometric mean of queries   :	$geomean_rounded"
		echo
		echo
		echo
	fi
done

