#!/bin/bash

# collects and calculates all performance-related numbers   
# as well as configuration info


if (( $# < 1 ))
then
  echo
  echo "usage:  $0  scope  scale_factor  mpssheap  mpssstack                   "
  echo "                                   run_name  seed                      "
  echo "                                   [num_streams  [num_disks            "
  echo "                                   [disk_size(GB)  [system_cost]]    "
  echo 
  echo "           - calculates and displays tpch metrics from the .out files  "
  echo "             in the current dir                                        "
  echo
  echo "           - also displays all essential information about the runtime "
  echo "             configuration:                                            "
  echo "                 System version                                       "
  echo "                 cpu type, clockspeed and count                        "
  echo "                 MPSS settings			                      "
  echo "                 disk configuration and RAID configuration            "
  echo
  exit
fi

scope=$1 


if [ $scope = throughput -o $scope = perf ]  
then
      if (( $# < 8 ))
      then
            echo
            echo "usage:  $0  $scope  scale_factor with_rf mpssheap  mpssstack     "
            echo "                                    run_name  seed        "
            echo "                                    num_streams           "
            echo
            exit
      fi 
fi


if [ $scope = power ]  
then
      if (( $# < 7 ))
      then
            echo
            echo "usage:  $0  $scope  scale_factor  with_rf mpssheap  mpssstack     "
            echo "                                   run_name  seed        "
            echo
            exit
      fi 
fi


if [ $scope = all ]
then
      if (( $# < 11 ))
      then
            echo
            echo "usage:  $0  $scope   scale_factor  with_rf mpssheap  mpssstack               "
            echo "                                   run_name  seed                    "
            echo "                                   num_streams  num_disks            "
            echo "                                   disk_size(GB)  system_cost        "
            echo 
            exit
      fi 
fi

       sf=$2    # scale_factor
  with_rf=$3 
 mpssheap=$4
mpssstack=$5
       rn=$6    # run_name
       sd=$7    # seed
       ns=$8    # num_streams
       nd=$9    # num_disks
       ds=${10}    # disk_size
       sc=${11} # system_price

#required for displaying stream0.out details.
if [ $scope = power ]
then
	ns=0
fi

echo " "
echo "                    TPC-H Report for $rn"
echo "                    ----------------------"
echo
uname -a
echo
show_machine
echo
date
echo
show_dbinfo > ${RESULTS}/dbinfo
echo
echo "MPSS Settings"
echo "-------------"
echo "MPSSHEAP = $mpssheap"
echo "MPSSSTACK = $mpssstack"
echo

echo


# any one of the folowing 3 statements do a form feed
#    perl -e 'print "\f" '  # double quotes intreprets backslash escapes
#    echo -e  "\f"       # -e turns on backslash escape interpretation
#    print "\f"      # print interprets backslash escapes by default

echo
echo
echo

echo
if [ $scope = power -o $scope = all -o $scope = perf ]
then
	errorpresent=$(grep -i "SQL Anywhere Error -[0-9]" $RESULTS/stream0.out)	

	if [ ! -z "$errorpresent" ]
	then
		echo
		echo "      stream0.out contains ERRORS. Unable to continue"
		echo
		#exit 1
	fi

	echo 
	geo_mean=`geo_mean_power_test.sh $with_rf`
      	echo "Power Test Geometric Mean                          `round_to_tenths.pl $geo_mean` "
fi


if [ $scope = throughput -o $scope = all -o $scope = perf ]
then
	for i in `seq 1 $ns`   # gets every streamN.out except stream0.out            
	do
		errorpresent=$(grep -i "SQL Anywhere Error -[0-9]" $RESULTS/stream${i}.out)	

		if [ ! -z "$errorpresent" ]
		then
			echo
			echo "      Stream$i contains ERRORS. Unable to continue"
			echo
			#exit 1
		fi
	done

	echo 
	amtt=`tpch_throughput_arithmetic_mean.sh $ns`
	echo "Throughput Test Arithmetic Mean                    $amtt " 

fi


echo 
echo 
echo 
echo 
echo "			   Numerical Quantities "
echo "			   -------------------- "
echo 
      echo "Database Scale Factor                        ${sf}GB"
      echo " "

if [ $scope = all ]
then
      sr=`data_storage_ratio.sh   $sf $nd $ds 1`
      echo "Total Data Storage / Database Size           $sr "
      echo " "
fi

if [ $scope = all ]
then
      show_load_time.sh
fi


if [ $scope = throughput -o $scope = all -o $scope = perf ] 
then
      echo "Query Streams for Throughput Test            $ns"
      echo " "
fi



if [ $scope = power -o $scope = all -o $scope = perf ]
then
	power=`tpch_power.sh  $sf $with_rf`        # rounded to tenths
	echo "TPC-H Power@${sf}GB                              $power" 
	echo " "
fi

if [ $scope = throughput -o $scope = all -o $scope = perf ]
then
      throughput=`tpch_throughput.sh  $sf  $with_rf $ns`     # rounded to tenths
      echo "TPC-H Throughput@${sf}GB                         $throughput" 
      echo " "
fi


if [ $scope = all -o $scope = perf ]
then
	line="sqrt($power*$throughput)" 
	# echo $line  [debug tpch_report.sh]

	c=$(echo "scale=3;$line"|bc)       #evaluate the composite metric"
	composite=`round_to_tenths.pl $c`
	echo "TPC-H Composite (QphH@${sf}GB)                   $composite"    # rounded to tenths
	echo " "
	if [ $scope = all ]
	then
		echo "Total System Cost Over 3 Years               $sc"
		echo " "
		pp=$(echo "scale=0;($sc/$composite)"|bc)
		echo "TPC-H Price/Performace Metric                $pp"
		echo " "
	fi
fi


if [ $scope = throughput -o $scope = all -o $scope = perf ]
then
	thi=`tpch_throughput_interval.sh $ns $with_rf`
	echo "TPC-H Throughput Interval:                   $thi seconds"
	echo 

fi


echo
echo
echo "			Summary Report"
echo "			--------------"
echo
echo


if [ $scope = throughput -o $scope = all -o $scope = perf -o $scope = power ]
then
      echo 
      echo " "
      echo "Stream Execution Durations:"
      echo
      tpch_stream_durations.sh  $scope all $ns $with_rf   
fi
      
if [ $with_rf -ne 0  -o $scope = refresh ]
then
	echo
	echo
	show_refresh_start_stop_times $scope all $ns
	show_refresh_inserts_deletes $scope $ns
fi

if [ ! $scope = refresh ]
then
	echo
	echo 
	echo "QUERY Execution times"
	echo "--------------------"
	echo
	extract_query_times $scope $ns

	echo
	echo
	echo "QUERY Execution Sequence"
	echo "-----------------------"
	get_query_sequence.sh $scope $ns
fi

echo
echo
echo "			Detailed Report"
echo "			--------------"
echo
echo


tpch_complete_report.sh $scope $sf $with_rf $rn $sd $ns >$TEMPFILES/complete_report.out
cat $TEMPFILES/complete_report.out

if [ $scope = all ]
then
	tpch_audit_report $sf $rn $ns 
fi
