
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Created By: David Walrath
% Create Date: 7/15/1999   
%                          
% This script creates various functions used by the Acid tests.
%                          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

print 'Creating history table'

allow error -141
execute { drop table history }
allow no error

execute {
create table history (
   h_p_key    unsigned INT NOT NULL ,
   h_s_key    unsigned INT NOT NULL ,
   h_o_key    unsigned INT NOT NULL ,
   h_l_key    INT NOT NULL,
   h_delta    INT NOT NULL,
   h_date_t   TIMESTAMP NOT NULL)
}

commit
execute {checkpoint}
print 'history table created'
print ' '


print 'creating the sleep procedure'

allow error -265
execute { DROP PROCEDURE dbo.sleep}
allow no error

execute{ create procedure dbo.sleep(in sleep_time integer default null)
begin
  declare command varchar(255);
  select 'xp_cmdshell ''sleep '+str(sleep_time)+'''' into command;
  execute immediate command
end;
} 

print 'creating the Acid Transaction'

allow error -265
execute { DROP PROCEDURE acid_transaction }
allow no error

execute{ CREATE PROCEDURE acid_transaction(
                IN  o_key      INT,
                IN  l_key      INT,
                IN  delta      INT,
                OUT rprice     Numeric(18,8),
                OUT quantity   INT,
                OUT tax        Numeric(18,8),
                OUT disc       Numeric(18,8),
                OUT extprice   Numeric(18,8),
                OUT ototal     Numeric(18,8)
                )
ON EXCEPTION RESUME
BEGIN
 DECLARE pkey           INT ;
 DECLARE skey           INT ;
 DECLARE cost           NUMERIC(18,8) ;
 DECLARE new_extprice   NUMERIC(18,8) ;
 DECLARE new_ototal     NUMERIC(18,8) ;
 DECLARE new_quantity   INT ;
 DECLARE c_sqlstate     char(5); 
 DECLARE num            INT ;

 LOOP1: LOOP

 COMMIT;

 acid1:
 BEGIN ATOMIC

   SELECT o_totalprice
      INTO ototal
      FROM orders
     WHERE o_orderkey = o_key ;
    SELECT l_quantity, 
           l_extendedprice, 
           l_partkey, 
           l_suppkey, 
           l_tax, 
           l_discount
      INTO quantity, 
           extprice,  
           pkey,  
           skey,      
           tax, 
           disc 
      FROM lineitem
     WHERE l_orderkey   = o_key 
       AND l_linenumber = l_key;
    -- CLEAN UP IMPRECICE NUMBERS
    SET ototal = ototal - "TRUNCATE"("truncate"(extprice*(1-disc),2)*(1+tax),2);
    SET rprice = "TRUNCATE"((extprice  / quantity),2);
    SET cost = "TRUNCATE"((rprice * delta),2);
    SET new_extprice = extprice + cost;
    SET new_ototal = "TRUNCATE"(new_extprice * (1.0 - disc),2);
    SET new_ototal = "TRUNCATE"(new_ototal * (1.0 + tax),2);
    SET new_ototal = ototal + new_ototal ;
    SET new_quantity = quantity + delta ;

    --
    -- Update LineItem
    --
    UPDATE lineitem 
       SET l_quantity      = new_quantity,
           l_extendedprice = new_extprice
     WHERE l_orderkey=o_key 
       AND l_linenumber=l_key;
    SELECT SQLSTATE INTO c_sqlstate;
    IF c_sqlstate = '00000' THEN
        --
        -- Update Orders
        --
        UPDATE orders
           SET o_totalprice = new_ototal 
         WHERE o_orderkey = o_key; 
        SELECT SQLSTATE INTO c_sqlstate;
        IF c_sqlstate = '00000'  THEN
            INSERT INTO history VALUES ( pkey, skey, o_key, l_key, delta, now(*)) ;
            SELECT SQLSTATE INTO c_sqlstate;
            IF c_sqlstate = '00000'
      	    	then message 'Completed ',o_key,' .......'; 
	     END IF;
	END IF;
    END IF;

 END acid1;

 if c_sqlstate = '00000'
 then LEAVE LOOP1;
 end if;

 select cast( rand()*4.5 as int) into num;
 message 'rollback sleep=', num,' sqlstate=',c_sqlstate;
 call dbo.sleep(num);

END LOOP LOOP1;

 RETURN(0);
END;
}

print 'Acid transaction created'
print ' '

print 'Creating Acid query'

allow error -265
execute { DROP PROCEDURE acid_single_query }
allow no error

execute{
CREATE PROCEDURE acid_single_query(
IN o_key INT,
OUT o_total NUMERIC(26,16) )
BEGIN
    SELECT 
	sum ("truncate" ("truncate"(
			  round(cast(l_extendedprice as numeric(26,16)),2) * 
			  (1 - round(cast(l_discount as numeric(26,16)),2)),2)
		    * (1 + round(cast(l_tax as numeric(26,16)),2)) , 2)) into o_total
    FROM lineitem WHERE l_orderkey = o_key;
 END
}

print 'Acid query created'
print ' '

print 'Creating Generate_acid_values function'

allow error -265
execute { DROP PROCEDURE generate_acid_values }
allow no error

execute{
create procedure generate_acid_values(
out orderkey 	int,
out linenumber	int,
out delta 	int)

BEGIN

 declare seed 	 	bigint;
 declare rand_dbl 	double precision;
 declare rand_int 	int;
 declare out_key 	int;

 declare times cursor for select datediff(millisecond,convert(char(10),getdate(), 116),now(*));
 declare random1 cursor for select rand(seed);
 declare random cursor for select rand();
 declare get_order cursor for 
	select o_orderkey from orders where o_orderkey = rand_int;
 declare get_linenumber cursor for 
	select max(l_linenumber) from lineitem 
	where l_orderkey = orderkey;

 open times;
 fetch next times into seed;
 open random1;
 fetch next random1 into rand_dbl;

 set out_key = 0;
 loop1:
 while out_key = 0 LOOP 
 	open random;
 	open get_order;

	fetch next random into rand_dbl;
	set rand_int = rand_dbl * 6001215 +1;
	fetch next get_order into out_key;

	close random;
	close get_order;
 end loop loop1;

 set orderkey = out_key;

 open get_linenumber;
 fetch next get_linenumber into linenumber;
 close get_linenumber;

 open random;
 fetch next random into rand_dbl;
 set delta = rand_dbl * 100 + 1;
 close random;

END
}
commit

print 'Generate_acid_values function created'
print ' '

print 'Creating Generate_Ps_Values function'

allow error -265
execute { DROP PROCEDURE generate_ps_values }
allow no error

execute{
create procedure generate_ps_values(
out partkey 	int,
out suppkey	int)

BEGIN

 declare seed 	 	bigint;
 declare rand_dbl 	double precision;
 declare rand_int 	int;
 declare out_key 	int;
 declare counter 	int;

 declare times cursor for select datediff(millisecond,convert(char(10),getdate(), 116),now(*));
 declare random1 cursor for select rand(seed);
 declare random cursor for select rand();
 declare get_supp cursor for 
	select ps_suppkey from partsupp 
	where ps_suppkey = rand_int;
 declare get_part cursor for 
	select ps_partkey from partsupp 
	where ps_suppkey = suppkey;

 open times;
 fetch next times into seed;
 open random1;
 fetch next random1 into rand_dbl;
 close random1;
 
 set out_key = 0;
 while out_key = 0 LOOP 
 	open random;
 	open get_supp ;

	fetch next random into rand_dbl;
	set rand_int = rand_dbl * 10000 +1;
	fetch next get_supp into out_key;

 	close random;
 	close get_supp ;
 end loop;
 set suppkey = out_key;

 set out_key = 0;
 set counter  = 0;
 open random;
 open get_part;
 fetch next random into rand_dbl;
 set rand_int = rand_dbl * 10 +1;

 loop1:
 while counter < rand_int LOOP 
	set counter = counter+1;
	fetch next get_part into out_key;
 end loop loop1;

 set partkey = out_key;
 close random;
 close get_part;

END
}
commit

print 'Generate_Ps_Values function created'
print ' '



print 'Creating Generate_acid_values2 function'

allow error -265
execute { DROP PROCEDURE generate_acid_values2 }
allow no error

execute{
create procedure generate_acid_values2(
in  streams 	int,
in  txns	int
)

BEGIN

 declare seed 	 	int;
 declare rand_dbl 	double precision;
 declare rand_int 	int;
 declare i int;
 declare j int;

 declare times cursor for 
	select datediff(millisecond,convert(char(10),getdate(), 116),now(*));
 declare random1 cursor for select rand(seed);
 declare random cursor for select rand();

 open times;
 fetch next times into seed;
 close times;

 open random1;
 fetch next random1 into rand_dbl;

 set i=1;
 set j=1;
 loop1:
 while i < streams LOOP 
   loop2:
    while j < txns LOOP 
 
	insert into acid_table (stream,seqnum) values (i,j);

   end loop loop2;
 end loop loop1;
 commit;

 update acid_table 
	set line=cast(rand(rowid(acid_table)+seed)*1500000+1 as int);
 commit;
 update acid_table 
	set okey=o_orderkey 
	from orders where o_orderkey=line;
 update acid_table 
	set delta=cast(rand(line)*100+1 as int);
 update acid_table 
	set line=max(l_linenumber) 
	from lineitem where l_orderkey=ordr;
 commit;


END
}
commit

print 'Generate_acid_values function2 created'
print ' '

