#!/bin/bash

if [ ${RESULTS} = "" ]
then
        echo "Please provide $RESULTS dir."
        exit
fi

if [ -z "$1" ]
then
	echo "Usage: gather_stats.sh <mode/query_number>."
	exit
fi

Platform=`uname -s `

if [ $1 = load ]
then
	iostat_out_path=$(echo ${RESULTS}/iostat.out)
	vmstat_out_path=$(echo ${RESULTS}/vmstat.out)
	mpstat_out_path=$(echo ${RESULTS}/mpstat.out)
	mpstat_percpu_out_path=$(echo ${RESULTS}/mpstat_percpu.out)
	top_out_path=$(echo ${RESULTS}/top.out)
	sar_out_path=$(echo ${RESULTS}/sar.out)
	iqmon_out_path=$(echo ${RESULTS})
	iqmon_suffix="load"
	timing_interval=1
elif [ $1 = power ]
then
	iostat_out_path=$(echo ${RESULTS}/iostat_power.out)
	vmstat_out_path=$(echo ${RESULTS}/vmstat_power.out)
	mpstat_out_path=$(echo ${RESULTS}/mpstat_power.out)
	mpstat_percpu_out_path=$(echo ${RESULTS}/mpstat_percpu_power.out)
	sar_out_path=$(echo ${RESULTS}/sar_power.out)
	#top_out_path=$(echo ${RESULTS}/top_power.out)
	iqmon_out_path=$(echo ${RESULTS})
	iqmon_suffix="power"
	timing_interval=1
elif [ $1 = throughput ]
then
	iostat_out_path=$(echo ${RESULTS}/iostat_throughput.out)
	vmstat_out_path=$(echo ${RESULTS}/vmstat_throughput.out)
	mpstat_out_path=$(echo ${RESULTS}/mpstat_throughput.out)
	mpstat_percpu_out_path=$(echo ${RESULTS}/mpstat_percpu_throughput.out)
	sar_out_path=$(echo ${RESULTS}/sar_throughput.out)
	top_out_path=$(echo ${RESULTS}/top_throughput.out)
	iqmon_out_path=$(echo ${RESULTS})
	iqmon_suffix="throughput"
	timing_interval=10
elif [ $1 = refresh ]
then
	iostat_out_path=$(echo ${RESULTS}/iostat_refresh.out)
	vmstat_out_path=$(echo ${RESULTS}/vmstat_refresh.out)
	mpstat_out_path=$(echo ${RESULTS}/mpstat_refresh.out)
	mpstat_percpu_out_path=$(echo ${RESULTS}/mpstat_percpu_refresh.out)
	sar_out_path=$(echo ${RESULTS}/sar_refresh.out)
	#top_out_path=$(echo ${RESULTS}/top_refresh.out)
	iqmon_out_path=$(echo ${RESULTS})
	iqmon_suffix="refresh"
	timing_interval=1
else
	if [ -z "$2" ]
	then
		iostat_out_path=$(echo ${RESULTS}/IOSTAT/q$1.out)
		vmstat_out_path=$(echo ${RESULTS}/VMSTAT/q$1.out)
		mpstat_out_path=$(echo ${RESULTS}/MPSTAT/q$1.out)
		mpstat_percpu_out_path=$(echo ${RESULTS}/MPSTAT/q$1_percpu.out)
		sar_out_path=$(echo ${RESULTS}/SAR/q$1.out)
		iqmon_out_path=$(echo ${RESULTS}/IQMON/)
		iqmon_suffix="q$1"
		#top_out_path=$(echo ${RESULTS}/TOP/$1.out)
	else
		iostat_out_path=$(echo ${RESULTS}/IOSTAT/$1_$2.out)
		vmstat_out_path=$(echo ${RESULTS}/VMSTAT/$1_$2.out)
		mpstat_out_path=$(echo ${RESULTS}/MPSTAT/$1_$2.out)
		mpstat_percpu_out_path=$(echo ${RESULTS}/MPSTAT/$1_$2_percpu.out)
		sar_out_path=$(echo ${RESULTS}/SAR/$1_$2.out)
		iqmon_out_path=$(echo ${RESULTS}/IQMON/$2/)
		iqmon_suffix="$1"

		mkdir -p ${RESULTS}/IQMON/$2
	fi

	timing_interval=1
fi	

if [ -f $TEMPFILES/stats_spids ]
then
	for i in `cat $TEMPFILES/stats_spids`
	do
        	kill -9 $i
	done
	rm -f $TEMPFILES/stats_spids
fi

if [ $Platform = Linux ]
then
        echo "user =`whoami`"
        echo "vmstat  $timing_interval"
	vmstat  $timing_interval | $TPCH_ROOT/bin/timestamp.pl >> $vmstat_out_path &
	echo $! >> $TEMPFILES/stats_spids
        
	mpstat $timing_interval >> $mpstat_out_path  &
	echo $! >> $TEMPFILES/stats_spids

	iostat -m -x -t $timing_interval | awk '{if (($4 > 0) || ($5 > 0) || ($1 == "Time:")) print $0 }' >> $iostat_out_path &
 	echo $! >> $TEMPFILES/stats_spids

	mpstat -P ALL $timing_interval >> $mpstat_percpu_out_path &
	echo $! >> $TEMPFILES/stats_spids
# Disable SAR output since it is space consuming
#	sar -A $timing_interval 10000000 -o $sar_out_path > /dev/null &
#	echo $! >> $TEMPFILES/stats_spids

	if [ $1 = load ]
	then
	top --d 2 -bcisSH >> $top_out_path &
	echo $! >> $TEMPFILES/stats_spids
	fi

elif [ $Platform = AIX ]
then
        vmstat -I -t $timing_interval >> $vmstat_out_path &
        echo $! > $TEMPFILES/stats_spids

        mpstat $timing_interval >> $mpstat_out_path  &
        echo $! >> $TEMPFILES/stats_spids

        iostat -DlVT -s $timing_interval >> $iostat_out_path &
	echo $! >> $TEMPFILES/stats_spids
elif [ $Platform = SunOS ]
then
  	vmstat $timing_interval >> $vmstat_out_path &
	echo $! > $TEMPFILES/stats_spids

	sar $timing_interval 1000000 >> $mpstat_out_path  &
	echo $! >> $TEMPFILES/stats_spids

	mpstat $timing_interval >> $mpstat_percpu_out_path  &
	echo $! >> $TEMPFILES/stats_spids

	iostat -z -xncXTd -P -M $timing_interval >> $iostat_out_path &
	echo $! >> $TEMPFILES/stats_spids
fi

	# collect IQ Monitor output for detailed server statistics (Prefetch, Cache)
$ISQL <<EOF &
declare local temporary table dummy_monitor_debug (dummy_column integer)
go
declare local temporary table dummy_monitor1_debug (dummy_column integer)
go
set option Monitor_Output_Directory = "${iqmon_out_path}"
go
iq utilities main into dummy_monitor_debug start monitor '-debug -interval $timing_interval -file_suffix ${iqmon_suffix}-debug.iqmon'
go
iq utilities private into dummy_monitor1_debug start monitor '-debug -interval $timing_interval -file_suffix ${iqmon_suffix}-debug.iqmon'
go
waitfor delay "10:00:00"
go
EOF
	echo $! >> $TEMPFILES/stats_spids


	# collect IQ Monitor output for Disk I/O
$ISQL <<EOF &
declare local temporary table dummy_monitor_io (dummy_column integer)
go
declare local temporary table dummy_monitor1_io (dummy_column integer)
go
set option Monitor_Output_Directory = "${iqmon_out_path}"
go
iq utilities main into dummy_monitor_io start monitor '-io -interval $timing_interval -file_suffix ${iqmon_suffix}-io.iqmon'
go
iq utilities private into dummy_monitor1_io start monitor '-io -interval $timing_interval -file_suffix ${iqmon_suffix}-io.iqmon'
go
waitfor delay "10:00:00"
go
EOF
        echo $! >> $TEMPFILES/stats_spids


	# collect IQ Monitor output for buffer cache
$ISQL <<EOF &
declare local temporary table dummy_monitor_cache (dummy_column integer)
go
declare local temporary table dummy_monitor1_cache (dummy_column integer)
go
set option Monitor_Output_Directory = "${iqmon_out_path}"
go
iq utilities main into dummy_monitor_cache start monitor '-cache -interval $timing_interval -file_suffix ${iqmon_suffix}-cache.iqmon'
go
iq utilities private into dummy_monitor1_cache  start monitor '-cache -interval $timing_interval -file_suffix ${iqmon_suffix}-cache.iqmon'
go
waitfor delay "10:00:00"
go
EOF
        echo $! >> $TEMPFILES/stats_spids

# Collect server stacks at regular intervals for LOAD (useful for debugging slow phases or system time)
#if [[ ( $1 = load || $1 = power || $1 = throughput ) && ( $PSTACK_ENABLE = "1" ) ]]
if [ $PSTACK_ENABLE = "1" ]
then
	collect_stack.sh ${RESULTS}/PSTACKS/ ${PSTACK_INTERVAL} &
        echo $! >> $TEMPFILES/stats_spids
fi
