#!/bin/ksh

Usage()
{
        clear
        echo
        echo "Usage: generate_data.sh [generate_data_files(1/0)] [ LINEITEM_PARTS [ ORDERS_PARTS [ PARTSUPP_PARTS [ PARTS_PARTS ] ] ] ]"
        echo
        echo "This script does the following:"
        echo
        echo "       [If you have set generate_data_files flag then]"
        echo "         - Generates load data files in \$DATA_DIR for all tables with number of file slices depending on script parameters or number of cores."
        echo
        echo "         - Generates 3*\$STREAM_COUNT Refresh Streams in \$DATA_DIR."
        echo
        echo "       [Always]"
        echo "         - Generates load sql files in \$TPCH_ROOT/load dir."
        echo
        echo "Make sure bm_setup script is run with all variables set appropriately."
        echo -en "Do You want to continue <Y/N>? "
        ReadYN || exit
}


ReadYN()
{
while :
do
        read ANS

        case "$ANS" in
                Y*|y*)  return 0;;
                N*|n*)  return 1;;
        esac

        echo -e "\nPlease answer 'y' or 'n'\n"
	
	echo -en "<Y/N>? ";
done
}

Usage

if [ -z "$DATA_DIR" -o -z "$TPCH_ROOT" -o -z "$SCALE_FACTOR" ]
then
        echo -e "One of the environment variable from DATA_DIR, TPCH_ROOT, SCALE_FACTOR is not defined. \nPlease make sure bm_setup has all variable set and is run."
        exit 1
fi


# auto-detect number of cores
if [ "`uname -s`" == "Linux" ]
then
	NUM_CORES=`cat /proc/cpuinfo | grep processor | wc -l`
elif [ "`uname -s`" == "AIX" ]
then
        NUM_CORES=`bindprocessor -q | awk '{print $NF}'`
        NUM_CORES=`expr $NUM_CORES + 1`
elif [ "`uname -s`" == "SunOS" ]
then
	NUM_CORES=`/usr/sbin/psrinfo | wc -l`
else
	NUM_CORES=1
fi


#The first argument is the flag indicating whether data should be generated or not
GENERATE_DATA_FILES=${1:-0};


LINEITEM_PARTS=${2:-`expr $NUM_CORES / 2`}
ORDERS_PARTS=${3:-`expr $NUM_CORES / 4`}
PARTSUPP_PARTS=${4:-`expr $NUM_CORES / 6`}
PARTS_PARTS=${5:-`expr $NUM_CORES / 12`}
# generate the input data files per table

if [ ${GENERATE_DATA_FILES} == "1" ]
then

	if [ ! -f $TPCH_ROOT/dbgen_src/qgen ]
        then
		echo "qgen not found. Please build qgen program in $TPCH_ROOT/dbgen_src"
		exit
        fi
        if [ ! -f $TPCH_ROOT/dbgen_src/dbgen ]
        then
		echo "dbgen not found. Please build dbgen program in $TPCH_ROOT/dbgen_src"
		exit
        fi


	cd $DATA_DIR
	cp ${TPCH_ROOT}/dbgen_src/dists.dss .
	if [ -f lineitem.tbl.1 -o -f lineitem.tbl -o -f orders.tbl.1 -o -f orders.tbl -o -f customer.tbl -o -f nation.tbl -o -f region.tbl -o -f supplier.tbl -o -f part.tbl.1 -o -f part.tbl -o -f partsupp.tbl.1 -o -f partsupp.tbl ]
	then
		echo -en "Some of tbl files are already present in \$DATA_DIR. Do you want to overwrite? <Y/N> "
		ReadYN || exit
		rm -rf *.tbl*
	fi

	dbgen -s ${SCALE_FACTOR} -T l &				#generate nation/region ONLY
	dbgen -s ${SCALE_FACTOR} -T s &				#generate suppliers ONLY
	dbgen -s ${SCALE_FACTOR} -T c &				#generate customers ONLY

	if [ ${LINEITEM_PARTS} -le 1 ]
	then
		LINEITEM_PARTS=1
		dbgen -s ${SCALE_FACTOR} -T L &
	else
		for i in `seq 1 ${LINEITEM_PARTS}`; do
		dbgen -s ${SCALE_FACTOR} -T L -C $LINEITEM_PARTS -S ${i} &	#generate lineitem ONLY 
		done
	fi


	if [ $ORDERS_PARTS -le 1 ]
	then
		ORDERS_PARTS=1
		dbgen -s ${SCALE_FACTOR} -T O &
	else
		for i in `seq 1 ${ORDERS_PARTS}`; do
		dbgen -s ${SCALE_FACTOR} -T O -C $ORDERS_PARTS -S ${i} &	#generate orders ONLY
		done
	fi

	if [ $PARTSUPP_PARTS -le 1 ]
	then
		PARTSUPP_PARTS=1
		dbgen -s ${SCALE_FACTOR} -T S &
	else
		for i in `seq 1 ${PARTSUPP_PARTS}`; do
		dbgen -s ${SCALE_FACTOR} -T S -C $PARTSUPP_PARTS -S ${i} &	#generate partsupp ONLY
		done
	fi

	if [ $PARTS_PARTS -le 1 ]
	then
		PARTS_PARTS=1
		dbgen -s ${SCALE_FACTOR} -T P &
	else
		for i in `seq 1 ${PARTS_PARTS}`; do
		dbgen -s ${SCALE_FACTOR} -T P -C $PARTS_PARTS -S ${i} &		#generate parts ONLY
		done
	fi

	# generate update/refresh streams count = stream_count * 3
	dbgen -s ${SCALE_FACTOR} -U $(($STREAM_COUNT * 3))
	wait
	cd -
fi

# lets create the load SQL for each table in database ...

# generate load sql for 'nation' table
cat > ${TPCH_ROOT}/load/load_nation.sql <<EOF
LOAD TABLE NATION (
N_NATIONKEY                                 '|',
N_NAME                                      '|',
N_REGIONKEY                                 '|',
N_COMMENT                                   '|'
)
FROM '${DATA_DIR}/nation.tbl'
escapes off
quotes off
row delimited by '\x0a'
WITH CHECKPOINT ON;
commit;
EOF

# generate load sql for 'region' table
cat > ${TPCH_ROOT}/load/load_region.sql <<EOF
LOAD TABLE REGION (
R_REGIONKEY                                 '|',
R_NAME                                      '|',
R_COMMENT                                   '|'
)
FROM '${DATA_DIR}/region.tbl'
escapes off
quotes off
row delimited by '\x0a'
WITH CHECKPOINT ON;
commit;
EOF

# generate load sql for 'supplier' table
cat > ${TPCH_ROOT}/load/load_supplier.sql <<EOF
LOAD TABLE SUPPLIER (
S_SUPPKEY                                   '|',
S_NAME                                      '|',
S_ADDRESS                                   '|',
S_NATIONKEY                                 '|',
S_PHONE                                     '|',
S_ACCTBAL                                   '|',
S_COMMENT                                   '|'
)
FROM '${DATA_DIR}/supplier.tbl'
escapes off
quotes off
row delimited by '\x0a'
WITH CHECKPOINT ON;
commit;
EOF

# generate load sql for 'customer' table
cat > ${TPCH_ROOT}/load/load_customer.sql <<EOF
LOAD TABLE CUSTOMER (
C_CUSTKEY                                   '|',
C_NAME                                      '|',
C_ADDRESS                                   '|',
C_NATIONKEY                                 '|',
C_PHONE                                     '|',
C_ACCTBAL                                   '|',
C_MKTSEGMENT                                '|',
C_COMMENT                                   '|'
)
FROM '${DATA_DIR}/customer.tbl'
escapes off
quotes off
row delimited by '\x0a'
WITH CHECKPOINT ON;
commit;
EOF

# generate load sql for 'lineitem' table
cat > ${TPCH_ROOT}/load/load_lineitem.sql <<EOF
LOAD TABLE LINEITEM (
L_ORDERKEY                                  '|',
L_PARTKEY                                   '|',
L_SUPPKEY                                   '|',
L_LINENUMBER                                '|',
L_QUANTITY                                  '|',
L_EXTENDEDPRICE                             '|',
L_DISCOUNT                                  '|',
L_TAX                                       '|',
L_RETURNFLAG                                '|',
L_LINESTATUS                                '|',
L_SHIPDATE                                  '|',
L_COMMITDATE                                '|',
L_RECEIPTDATE                               '|',
L_SHIPINSTRUCT                              '|',
L_SHIPMODE                                  '|',
L_COMMENT                                   '|'
)
FROM
EOF
#
for i in `seq 1 $((${LINEITEM_PARTS} - 1))`; do
cat >> ${TPCH_ROOT}/load/load_lineitem.sql <<EOF
'${DATA_DIR}/lineitem.tbl.${i}',
EOF
done
#
cat >> ${TPCH_ROOT}/load/load_lineitem.sql <<EOF
`if [ ${LINEITEM_PARTS} -eq 1 ]
then
	echo \'${DATA_DIR}/lineitem.tbl\'
else
        echo \'${DATA_DIR}/lineitem.tbl.${LINEITEM_PARTS}\'
fi`
escapes off
quotes off
row delimited by '\x0a'
WITH CHECKPOINT ON;
commit;
checkpoint;
commit;
EOF

# generate load sql for 'orders' table
cat > ${TPCH_ROOT}/load/load_orders.sql <<EOF
LOAD TABLE ORDERS (
O_ORDERKEY             			    '|',
O_CUSTKEY              			    '|',
O_ORDERSTATUS          			    '|',
O_TOTALPRICE           			    '|',
O_ORDERDATE            			    '|',
O_ORDERPRIORITY        			    '|',
O_CLERK                			    '|',
O_SHIPPRIORITY         			    '|',
O_COMMENT              			    '|'
)
FROM 
EOF
#
for i in `seq 1 $((${ORDERS_PARTS} - 1))`; do
cat >> ${TPCH_ROOT}/load/load_orders.sql <<EOF
'${DATA_DIR}/orders.tbl.${i}',
EOF
done
#
cat >> ${TPCH_ROOT}/load/load_orders.sql <<EOF
`if [ ${ORDERS_PARTS} -eq 1 ]
then
        echo \'${DATA_DIR}/orders.tbl\'
else
        echo \'${DATA_DIR}/orders.tbl.${ORDERS_PARTS}\'
fi`
escapes off
quotes off
row delimited by '\x0a'
WITH CHECKPOINT ON;
commit;
EOF

# generate load sql for 'partsupp' table
cat > ${TPCH_ROOT}/load/load_partsupp.sql <<EOF
LOAD TABLE PARTSUPP (
PS_PARTKEY                		    '|',
PS_SUPPKEY                		    '|',
PS_AVAILQTY               		    '|',
PS_SUPPLYCOST             		    '|',
PS_COMMENT                		    '|'
)
FROM
EOF
#
for i in `seq 1 $((${PARTSUPP_PARTS} - 1))`; do
cat >> ${TPCH_ROOT}/load/load_partsupp.sql <<EOF
'${DATA_DIR}/partsupp.tbl.${i}',
EOF
done
#
cat >> ${TPCH_ROOT}/load/load_partsupp.sql <<EOF
`if [ ${PARTSUPP_PARTS} -eq 1 ]
then
        echo \'${DATA_DIR}/partsupp.tbl\'
else
        echo \'${DATA_DIR}/partsupp.tbl.${PARTSUPP_PARTS}\'
fi`
escapes off
quotes off
row delimited by '\x0a'
WITH CHECKPOINT ON;
commit;
EOF

# generate load sql for 'part' table
cat > ${TPCH_ROOT}/load/load_part.sql <<EOF
LOAD TABLE PART (
P_PARTKEY        			    '|',
P_NAME           			    '|',
P_MFGR           			    '|',
P_BRAND          			    '|',
P_TYPE           			    '|',
P_SIZE           			    '|',
P_CONTAINER      			    '|',
P_RETAILPRICE    			    '|',
P_COMMENT        			    '|'
)
FROM 
EOF
#
for i in `seq 1 $((${PARTS_PARTS} - 1))`; do
cat >> ${TPCH_ROOT}/load/load_part.sql <<EOF
'${DATA_DIR}/part.tbl.${i}',
EOF
done
#
cat >> ${TPCH_ROOT}/load/load_part.sql <<EOF
`if [ ${PARTS_PARTS} -eq 1 ]
then
        echo \'${DATA_DIR}/part.tbl\'
else
	echo \'${DATA_DIR}/part.tbl.${PARTS_PARTS}\'
fi`
escapes off
quotes off
row delimited by '\x0a'
WITH CHECKPOINT ON;
commit;
EOF
