if [ -z "$2" ]
then
        echo 'Usage: powertest.sh <SCOPE> <SCALE_FACTOR> [WITH_RF] [Enable_Monitoring]'
	echo 'Allowed values for scope Report, Run, All'
        exit 1
fi

SCOPE=$1
SCALE_FACTOR=$2
if [ -z "$3" ]
then
	WITH_RF=1	
else
	WITH_RF=$3
fi

if [ -z "$4" ]
then
       Enable_Monitoring=0
else
       Enable_Monitoring=$4
fi

CURRENTLY_RUNNING=power; export CURRENTLY_RUNNING

if [ $Enable_Monitoring -ne 0 ] && [ $DEBUG -eq 0 ]
then
        start_monitor.sh
fi

echo $SCOPE
if [ $SCOPE = 'Run' -o $SCOPE = 'All' ];
then
	##  Check power test is allowed or not				 ##
	##  It is allowed unconditionally if executed without_RFs              ##
	##  It is not allowed if RFs are executed more than or equal to	 ## 
	##  MAX_TIMES RFs are allowed to execute before RELOAD		 ##

	cp $STREAMS/stream_seeds $RESULTS
	if [ $WITH_RF -ne 0 ];
	then
		MAX_STREAMS=`ls ${DEV_DIR}/data/lineitem.tbl.u* 2>/dev/null| wc -l`
		LAST_STREAM=`cat ${DEV_DIR}/data/last_stream`

		if [ $LAST_STREAM -ge $MAX_STREAMS ];
			then
			echo "Please reload the data or generate more RF STREAMS as total Refresh STREAMS generated have exhausted."
			echo "Use instructions in README to generate additional RF STREAMS data using dbgen."
			exit 1
		fi
		gen_update_power.sh $LAST_STREAM
	fi 

	echo "  "
	echo "Start the Power Test   `date` "
	echo "  "

	if [ $WITH_RF -ne 0 ];
	then
		#
		# Touch the lock files so that the RF will pause after RF1 and wait for the query stream to complete
		#
		touch $TPCH_ROOT/rf1.lock
		touch $TPCH_ROOT/rf2.lock
		#
		# Start the RF Stream in the Background
		#

                LAST_STREAM=`expr $LAST_STREAM + 1`
                echo $LAST_STREAM > $DEV_DIR/data/last_stream


		$ISQL_W3 -i $GENERATED_FILES/update_power.sql > $RESULTS/update_power.out &
		rfspid=$!
		#
		# Wait for RF1 to Complete before Starting the Query Stream
		#
		# the rf1.lock file will be removed after RF1 completes
		# RF will then wait for rf2.lock to be removed before continuing
		#
		while [ -f $TPCH_ROOT/rf1.lock ]
		do
			# Sleep while Trigger file exists
			# echo "Waiting until trigger file is removed"
			echo Waiting on rf1 to finish
			sleep 1 
		done
		#
		# Continue RF1 has completed and RF2 is waiting on the Query Stream to Complete
		#
		echo "  "
	fi

	#
	# Start the Query Stream
	#
        echo "Start the Query Stream"
        echo "$ISQL_W -i $STREAMS/stream0.sql -e > $RESULTS/stream0.out"
	$ISQL_W -i $STREAMS/stream0.sql -e > $RESULTS/stream0.out 
       
        echo "Query Stream end"

	if [ $WITH_RF -ne 0 ];
	then
		#
		# Remove the lock file so that the RF will continue with RF2
		#
		`\rm -f $TPCH_ROOT/rf2.lock`
		#
		# Now wait for the RF stream 0 to complete
		#
	
		wait $rfspid
	fi

	echo End Power `date`
fi

if [ $Enable_Monitoring -ne 0 ] && [ $DEBUG -eq 0 ]
then
        kill_stats.sh
fi

if [ $SCOPE = 'Report' -o $SCOPE = 'All' ];
then
	tpch_report.sh power $SCALE_FACTOR $WITH_RF 0  0 power $SEED > $RESULTS/powertest.out
fi

os=$(uname)
if [ $os = AIX ]
then
	echo "After completion of the Power Test" >> $RESULTS/sysunused
	echo "The following volume groups are currently online" >> $RESULTS/sysunused
	echo `date` >> $RESULTS/sysunused
	echo>> $RESULTS/sysunused
	lsvg -o >> $RESULTS/sysunused
fi
