--------Create RF1--------------

CREATE PROCEDURE DBA.tpch_rf1 (IN data_directory varchar(128),
                               IN stream_number varchar(3))
ON EXCEPTION RESUME
BEGIN
  DECLARE sql_cmd long varchar;
  DECLARE c_lf varchar(2);
  DECLARE rf1_start timestamp;
  DECLARE rf1_stop timestamp;
  DECLARE n_seconds numeric(16,5);
  DECLARE cur_sqlstate CHAR(5);
  
  SET c_lf=char(10);
  SET rf1_start = dateformat(now(*) ,'yyyy-Mm-Dd hh:nn:ss.sss');
  SELECT 'Stream'||stream_number||' RF1 START TIME --, '||dateformat(rf1_start,'yyyy-mm-dd hh:nn:ss.sss');
 
  SET sql_cmd='load table orders ( '+c_lf;
  SET sql_cmd=sql_cmd+' o_orderkey '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' o_custkey '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' o_orderstatus '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' o_totalprice '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' o_orderdate date('+char(39)+'YYYY-MM-DD'+char(39)+'), filler(1), '+c_lf;
  SET sql_cmd=sql_cmd+' o_orderpriority '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' o_clerk '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' o_shippriority '+char(39)+'|'+char(39)+','+c_lf;
  SET sql_cmd=sql_cmd+' o_comment '+char(39)+'|'+char(39)+' ) '+c_lf;
  SET sql_cmd=sql_cmd+'from '+char(39)+data_directory+'orders.tbl.u'+stream_number+char(39)+c_lf;
  SET sql_cmd=sql_cmd+'row delimited by '+char(39)+'\\x0a'+char(39)+' quotes off escapes off preview on;';
  EXECUTE IMMEDIATE with quotes on sql_cmd;
  SELECT SQLSTATE INTO cur_sqlstate;
  IF cur_sqlstate != '00000' THEN 
    ROLLBACK;
    RAISERROR 23002 'RF1 failed while inserting into orders with SQLSTATE: ', cur_sqlstate;
    RETURN(1);
  END IF;

--------insert into lineitem------------
  SET sql_cmd='load table lineitem ( '+c_lf;
  SET sql_cmd=sql_cmd+' l_orderkey '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_partkey '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_suppkey '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_linenumber '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_quantity '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_extendedprice '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_discount '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_tax '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_returnflag '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_linestatus '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_shipdate date('+char(39)+'YYYY-MM-DD'+char(39)+'), filler(1), '+c_lf;
  SET sql_cmd=sql_cmd+' l_commitdate date('+char(39)+'YYYY-MM-DD'+char(39)+'), filler(1), '+c_lf;
  SET sql_cmd=sql_cmd+' l_receiptdate date('+char(39)+'YYYY-MM-DD'+char(39)+'), filler(1), '+c_lf;
  SET sql_cmd=sql_cmd+' l_shipinstruct '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_shipmode '+char(39)+'|'+char(39)+', '+c_lf;
  SET sql_cmd=sql_cmd+' l_comment '+char(39)+'|'+char(39)+' )'+c_lf;
  SET sql_cmd=sql_cmd+'from '+char(39)+data_directory+'lineitem.tbl.u'+stream_number+char(39)+c_lf;
  SET sql_cmd=sql_cmd+'row delimited by '+char(39)+'\\x0a'+char(39)+c_lf+'quotes off escapes off preview on;';
  EXECUTE IMMEDIATE with quotes on sql_cmd;
  
  SELECT SQLSTATE INTO cur_sqlstate;
  IF cur_sqlstate != '00000' THEN 
    rollback;
    RAISERROR 23002 'RF1 failed while inserting into lineitem with SQLSTATE: ', cur_sqlstate;
    RETURN(1);
  END IF;
  COMMIT;
  SET rf1_stop = dateformat(now(*) ,'yyyy-Mm-Dd hh:nn:ss.sss');
  SELECT 'Stream'||stream_number||' RF1 STOP TIME --, '||dateformat(rf1_stop ,'yyyy-mm-dd hh:nn:ss.sss');

---------Find the execution time of rf1.---------------
  SET n_seconds=cast(datediff(millisecond,rf1_start,rf1_stop) AS numeric(16,5))/1000;
  SET sql_cmd='Stream_'+stream_number+' RF1 Elapsed time --, '+cast(n_seconds AS varchar(20))+ ' seconds' ;
  SELECT sql_cmd;
  RETURN(0);
END;

---------Create RF2---------------------------
CREATE PROCEDURE DBA.tpch_rf2 (in data_directory varchar(128),
                               in stream_number varchar(3))
ON exception resume
BEGIN
  DECLARE sql_cmd long varchar;
  DECLARE c_lf varchar(2);
  DECLARE rf2_start timestamp;
  DECLARE rf2_stop timestamp;
  DECLARE n_seconds numeric(16,5);
  DECLARE cur_sqlstate CHAR(5);

  SET c_lf=char(10);
  SET rf2_start = dateformat(now(*) ,'yyyy-Mm-Dd hh:nn:ss.sss');
  SELECT 'Stream'||stream_number||' RF2 START TIME --, '||dateformat(rf2_start,'yyyy-mm-dd hh:nn:ss.sss');
   
  CREATE TABLE #delete_table ( d_orderkey UNSIGNED BIGINT );

  SET sql_cmd='load table #delete_table  (d_orderkey '+char(39)+'|'+char(39)+')'+c_lf;
  SET sql_cmd=sql_cmd+'from '+char(39)+data_directory+'delete.'+stream_number+char(39)+c_lf;
  SET sql_cmd=sql_cmd+'row delimited by '+char(39)+'\\x0a'+char(39)+c_lf;
  SET sql_cmd=sql_cmd+'quotes off '+c_lf;
  SET sql_cmd=sql_cmd+'escapes off; '+c_lf;
 
  EXECUTE IMMEDIATE with quotes on sql_cmd;
  SELECT SQLSTATE INTO cur_sqlstate;
  IF cur_sqlstate != '00000' THEN 
    ROLLBACK;
    SET sql_cmd='RF2 failed at Step 1 with SQLSTATE: '+cur_sqlstate;
    RAISERROR 23002 sql_cmd;
    RETURN(1);
  END IF;

-------- Delete from LineItem---------
  DELETE lineitem FROM lineitem 
     WHERE l_orderkey in (select d_orderkey from  #delete_table);
  SELECT SQLSTATE INTO cur_sqlstate;

  IF cur_sqlstate != '00000' THEN 
    ROLLBACK;
    SET sql_cmd='RF2 failed at Step 2 with SQLSTATE: '+cur_sqlstate;
    RAISERROR 23002 sql_cmd;
    RETURN(1);
  END IF;

-----Delete fom Orders.----------
  DELETE orders FROM orders 
      WHERE o_orderkey in (select d_orderkey from  #delete_table);
  SELECT SQLSTATE INTO cur_sqlstate;

  IF cur_sqlstate != '00000' THEN 
    ROLLBACK;
    SET sql_cmd='RF2 failed at Step 3 with SQLSTATE: '+cur_sqlstate;
    RAISERROR 23002 sql_cmd;
    RETURN(1);
  END IF;

  COMMIT;
  DROP TABLE #delete_table;
  SET rf2_stop = dateformat(now(*) ,'yyyy-Mm-Dd hh:nn:ss.sss');
  SELECT 'Stream'||stream_number||' RF2 STOP TIME --, '||dateformat(rf2_stop,'yyyy-mm-dd hh:nn:ss.sss');
--------- Calculate execution time for rf2.-------

  SET n_seconds=cast(datediff(millisecond,rf2_start,rf2_stop) as numeric(16,5))/1000;
  SET sql_cmd='Stream_'+stream_number+' RF2 Elapsed time --, '+cast(n_seconds as varchar(20))+ ' seconds' ;
  SELECT sql_cmd;
  RETURN(0);
END;
