
set temporary option on_error = 'continue';


set temporary option Allow_nulls_by_default='on';

CREATE TABLE region
  (
    r_regionkey            unsigned int,
    r_name                 char(25),
    r_comment              varchar(152),
    PRIMARY KEY (r_regionkey)
  );



CREATE TABLE nation
  (
    n_nationkey            unsigned int,
    n_name                 char(25),
    n_regionkey            unsigned int ,
    n_comment              varchar(152),
    PRIMARY KEY (n_nationkey)
  );



CREATE TABLE supplier
  (
    s_suppkey              unsigned int,
    s_name                 char(25),
    s_address              varchar(40),
    s_nationkey            unsigned int ,
    s_phone                char(15),
    s_acctbal              double precision,
    s_comment              varchar(101),
    PRIMARY KEY (s_suppkey)
  );



CREATE TABLE part
  (
    p_partkey              unsigned int,
    p_name                 varchar(55),
    p_mfgr                 char(25),
    p_brand                char(10),
    p_type                 varchar(25),
    p_size                 int,
    p_container            char(10),
    p_retailprice          double precision,
    p_comment              varchar(23),
    PRIMARY KEY(p_partkey)
  );



CREATE TABLE partsupp
  (
    ps_partkey             unsigned int ,
    ps_suppkey             unsigned int ,
    ps_availqty            integer,
    ps_supplycost          double precision,
    ps_comment             varchar(199),
    PRIMARY KEY (ps_partkey, ps_suppkey)
  );

CREATE TABLE customer
  (
    c_custkey              unsigned int,
    c_name                 varchar(25),
    c_address              varchar(40),
    c_nationkey            unsigned int ,
    c_phone                char(15),
    c_acctbal              double precision,
    c_mktsegment           char(10),
    c_comment              varchar(117),
    PRIMARY KEY(c_custkey)
  );



CREATE TABLE orders
  (
    o_orderkey             unsigned bigint,
    o_custkey              unsigned int ,
    o_orderstatus          char(1),
    o_totalprice           double precision,
    o_orderdate            date,
    o_orderpriority        char(15),
    o_clerk                char(15),
    o_shippriority         int,
    o_comment              varchar(79),
    PRIMARY KEY (o_orderkey)
  )partition by range (o_orderdate)
    ( 
      p0 values <= ('1991-12-31'),
      p1 values <= ('1992-12-31'),
      p2 values <= ('1993-12-31'),
      p3 values <= ('1994-12-31'),
      p4 values <= ('1995-12-31'),
      p5 values <= ('1996-12-31'),
      p6 values <= ('1997-12-31'),
      p7 values <= ('1998-12-31'),
      p8 values <= (MAX)
    );



CREATE TABLE lineitem
  (
    l_orderkey             unsigned bigint ,
    l_partkey              unsigned int,
    l_suppkey              unsigned int,
    l_linenumber           int,
    l_quantity             double precision,
    l_extendedprice        double precision,
    l_discount             double precision,
    l_tax                  double precision,
    l_returnflag           char(1),
    l_linestatus           char(1),
    l_shipdate             date,
    l_commitdate           date,
    l_receiptdate          date,
    l_shipinstruct         char(25),
    l_shipmode             char(10),
    l_comment              varchar(44)
  )
  partition by range(l_shipdate)
   ( 
    p0 values <= ('1991-12-31'),
    p1 values <= ('1992-12-31'),
    p2 values <= ('1993-12-31'),
    p3 values <= ('1994-12-31'),
    p4 values <= ('1995-12-31'),
    p5 values <= ('1996-12-31'),
    p6 values <= ('1997-12-31'),
    p7 values <= ('1998-12-31'),
    p8 values <= (MAX)
  );

--CREATE DATE INDEX o_orderdate_date ON orders(o_orderdate) ;
--CREATE DATE INDEX l_shipdate_date ON lineitem(l_shipdate) ;
--CREATE DATE INDEX l_receiptdate_date ON lineitem(l_receiptdate);

--create HG index n_regionkey_hg on nation ( n_regionkey );
--create HG index s_nationkey_hg on supplier ( s_nationkey );
--create HG index c_nationkey_hg on customer ( c_nationkey );
--create HG index o_custkey_hg on orders ( o_custkey );
--create HG index l_partkey_hg on lineitem ( l_partkey );
--
--create HG index ps_suppkey_hg on partsupp ( ps_suppkey );
--create HG index ps_partkey_hg on partsupp ( ps_partkey );
--
--create HG index l_suppkey_hg on lineitem ( l_suppkey );
--
--create HG index l_orderkey_hg on lineitem ( l_orderkey );

create HG index n_regionkey_hg on nation ( n_regionkey );
create HG index s_nationkey_hg on supplier ( s_nationkey );
create HG index c_nationkey_hg on customer ( c_nationkey );

create HG index l_orderkey_hg on lineitem ( l_orderkey );

create HG index o_custkey_hg on orders ( o_custkey );

create HG index ps_suppkey_hg on partsupp ( ps_suppkey );
create HG index ps_partkey_hg on partsupp ( ps_partkey );
--create HG index l_partkey_hg on lineitem ( l_partkey );
--create HG index l_suppkey_hg on lineitem ( l_suppkey );

commit;

