-- ==================================================================+
--  FILENAME
--       DBINSERT.SQL
--  DESCRIPTION
--       INSERTS DUPLICATE ROWS WITH NEW KEY VALUES AND
--       INSERTS ROWS WITH VALUES BEYOND THE TPC-H RANGES.
--
-- ==================================================================
--
--
-- ==================================================================
-- DUPLICATES
-- ==================================================================
-- GET TIMESTAMP
SELECT CONVERT(CHAR, GETDATE(), 120)
go
if exists (select name from sysobjects where name='TEMP_PART')
  drop table TEMP_PART
go
if exists (select name from sysobjects where name='TEMP_SUPPLIER')
  drop table TEMP_SUPPLIER
go
if exists (select name from sysobjects where name='TEMP_PARTSUPP')
  drop table TEMP_PARTSUPP
go
if exists (select name from sysobjects where name='TEMP_CUSTOMER')
  drop table TEMP_CUSTOMER
go
if exists (select name from sysobjects where name='TEMP_ORDERS')
  drop table TEMP_ORDERS
go
if exists (select name from sysobjects where name='TEMP_LINEITEM')
  drop table TEMP_LINEITEM
go
if exists (select name from sysobjects where name='TEMP_NATION')
  drop table TEMP_NATION
go
if exists (select name from sysobjects where name='TEMP_REGION')
 drop table TEMP_REGION
go
CREATE TABLE TEMP_PART
	 (P_PARTKEY             INT                     NOT NULL,
	  P_NAME                VARCHAR(55)             NOT NULL,
	  P_MFGR                CHAR(25)                NOT NULL,
	  P_BRAND               CHAR(10)                NOT NULL,
	  P_TYPE                VARCHAR(25)             NOT NULL,
	  P_SIZE                INT                     NOT NULL,
	  P_CONTAINER           CHAR(10)                NOT NULL,
	  P_RETAILPRICE         double precision        NOT NULL,
	  P_COMMENT             VARCHAR(23)             NOT NULL)
go
CREATE TABLE TEMP_SUPPLIER
	 (S_SUPPKEY             INT                    NOT NULL,
	  S_NAME                CHAR(25)               NOT NULL,
	  S_ADDRESS             VARCHAR(40)            NOT NULL,
	  S_NATIONKEY           INT                    NOT NULL,
	  S_PHONE               CHAR(15)               NOT NULL,
	  S_ACCTBAL             double precision       NOT NULL,
	  S_COMMENT             VARCHAR(101)           NOT NULL)
go
CREATE TABLE TEMP_PARTSUPP
	 (PS_PARTKEY    INT                    NOT NULL,
	  PS_SUPPKEY    INT                    NOT NULL,
	  PS_AVAILQTY   INT                    NOT NULL,
	  PS_SUPPLYCOST double precision       NOT NULL,
	  PS_COMMENT    VARCHAR(199)           NOT NULL)
go
CREATE TABLE TEMP_CUSTOMER
	 (C_CUSTKEY     INT                    NOT NULL,
	  C_NAME        VARCHAR(25)            NOT NULL,
	  C_ADDRESS     VARCHAR(40)            NOT NULL,
	  C_NATIONKEY   INT                    NOT NULL,
	  C_PHONE       CHAR(15)               NOT NULL,
	  C_ACCTBAL     double precision       NOT NULL,
	  C_MKTSEGMENT  CHAR(10)               NOT NULL,
	  C_COMMENT     CHAR(117)              NOT NULL)
go
CREATE TABLE TEMP_ORDERS
	 (O_ORDERKEY      INT                  NOT NULL,
	  O_CUSTKEY       INT                  NOT NULL,
	  O_ORDERSTATUS   CHAR(1)              NOT NULL,
	  O_TOTALPRICE    double precision     NOT NULL,
	  O_ORDERDATE     DATETIME             NOT NULL,
	  O_ORDERPRIORITY CHAR(15)             NOT NULL,
	  O_CLERK         CHAR(15)             NOT NULL,
	  O_SHIPPRIORITY  INT                  NOT NULL,
	  O_COMMENT       VARCHAR(79)          NOT NULL)
go
CREATE TABLE TEMP_LINEITEM
	(L_ORDERKEY               INT          NOT NULL,
	 L_PARTKEY                INT          NOT NULL,
	 L_SUPPKEY                INT          NOT NULL,
	 L_LINENUMBER             INT          NOT NULL,
	 L_QUANTITY               double precision    NOT NULL,
	 L_EXTENDEDPRICE          double precision    NOT NULL,
	 L_DISCOUNT               double precision    NOT NULL,
	 L_TAX                    double precision    NOT NULL,
	 L_RETURNFLAG             CHAR(1)      NOT NULL,
	 L_LINESTATUS             CHAR(1)      NOT NULL,
	 L_SHIPDATE               DATETIME     NOT NULL,
	 L_COMMITDATE             DATETIME     NOT NULL,
	 L_RECEIPTDATE            DATETIME     NOT NULL,
	 L_SHIPINSTRUCT           CHAR(25)     NOT NULL,
	 L_SHIPMODE               CHAR(10)     NOT NULL,
	 L_COMMENT                VARCHAR(44)  NOT NULL)
go
CREATE TABLE TEMP_NATION
	(N_NATIONKEY        INT          NOT NULL,
	 N_NAME             CHAR(25)     NOT NULL,
	 N_REGIONKEY        INT          NOT NULL,
	 N_COMMENT          VARCHAR(152) NOT NULL)
go
CREATE TABLE TEMP_REGION
	(R_REGIONKEY        INT          NOT NULL,
	 R_NAME             CHAR(25)     NOT NULL,
	 R_COMMENT          VARCHAR(152) NOT NULL)
go

BEGIN TRANSACTION;
go
INSERT INTO TEMP_PART 
SELECT * FROM PART
	WHERE P_PARTKEY = 1;
go
UPDATE TEMP_PART
	SET P_PARTKEY = 2147483647;
go
INSERT INTO PART
	SELECT * FROM TEMP_PART;
go
SELECT * FROM PART
	WHERE P_PARTKEY = 2147483647
	OR P_PARTKEY = 1;
go
DELETE FROM PART
	WHERE P_PARTKEY = 2147483647;
go
COMMIT;
go
BEGIN TRANSACTION;
go
INSERT INTO TEMP_SUPPLIER
SELECT * FROM SUPPLIER
	WHERE S_SUPPKEY = 1
go
UPDATE TEMP_SUPPLIER
	SET S_SUPPKEY = 2147483647;
go
INSERT INTO SUPPLIER
	SELECT * FROM TEMP_SUPPLIER;
go
SELECT * FROM SUPPLIER
	WHERE S_SUPPKEY = 2147483647
	OR S_SUPPKEY = 1;
go
DELETE FROM SUPPLIER
	WHERE S_SUPPKEY = 2147483647;
go
COMMIT;
go
BEGIN TRANSACTION;
go
INSERT INTO TEMP_PARTSUPP
SELECT * FROM PARTSUPP
	WHERE PS_PARTKEY = 1
	AND PS_SUPPKEY = 2;
go
UPDATE TEMP_PARTSUPP
	SET PS_PARTKEY = 2147483647,
	    PS_SUPPKEY = 2147483647;
go
INSERT INTO PARTSUPP
	SELECT * FROM TEMP_PARTSUPP;
go
SELECT * FROM PARTSUPP
	WHERE (PS_PARTKEY = 2147483647
	AND    PS_SUPPKEY = 2147483647)
	OR    (PS_PARTKEY = 1
	AND    PS_SUPPKEY = 2);
go
DELETE FROM PARTSUPP
	WHERE PS_PARTKEY = 2147483647
	AND   PS_SUPPKEY = 2147483647;
go
COMMIT;
go
BEGIN TRANSACTION;
go
INSERT INTO TEMP_CUSTOMER
SELECT * FROM CUSTOMER
	WHERE C_CUSTKEY = 1;
go
UPDATE TEMP_CUSTOMER
	SET C_CUSTKEY = 2147483647;
go
INSERT INTO CUSTOMER
	SELECT * FROM TEMP_CUSTOMER;
go
SELECT * FROM CUSTOMER
	WHERE C_CUSTKEY = 2147483647
	OR C_CUSTKEY = 1;
go
DELETE FROM CUSTOMER
	WHERE C_CUSTKEY = 2147483647;
go
COMMIT;
go
BEGIN TRANSACTION;
go
INSERT INTO TEMP_ORDERS
SELECT * FROM ORDERS
	WHERE O_ORDERKEY = (SELECT MIN(O_ORDERKEY) FROM ORDERS);
go
UPDATE TEMP_ORDERS
	SET O_ORDERKEY = 2147483647;
go
INSERT INTO ORDERS
	SELECT * FROM TEMP_ORDERS;
go
SELECT * FROM ORDERS
	WHERE O_ORDERKEY = 2147483647
	OR O_ORDERKEY = (SELECT MIN(O_ORDERKEY) FROM ORDERS);
go
DELETE FROM ORDERS
	WHERE O_ORDERKEY = 2147483647;
go
COMMIT;
go
BEGIN TRANSACTION;
go
INSERT INTO TEMP_LINEITEM
SELECT * FROM LINEITEM
	WHERE L_ORDERKEY = (SELECT MIN(O_ORDERKEY) FROM ORDERS)
	AND   L_LINENUMBER = 1;
go
UPDATE TEMP_LINEITEM
	SET L_ORDERKEY = 2147483647,
	    L_PARTKEY  = 2147483647,
	    L_SUPPKEY  = 2147483647,
	    L_LINENUMBER = -2147483646;
go
INSERT INTO LINEITEM
	SELECT * FROM TEMP_LINEITEM;
go
SELECT * FROM LINEITEM
	WHERE (L_ORDERKEY = 2147483647
	AND    L_PARTKEY  = 2147483647
	AND    L_SUPPKEY  = 2147483647
	AND    L_LINENUMBER = -2147483646)
	OR 	(L_ORDERKEY = (SELECT MIN(O_ORDERKEY) FROM ORDERS)
	AND    L_LINENUMBER = 1);
go
DELETE FROM LINEITEM
	WHERE L_ORDERKEY = 2147483647
	AND   L_PARTKEY  = 2147483647
	AND   L_SUPPKEY  = 2147483647
	AND   L_LINENUMBER = -2147483646;
go
COMMIT;
go
BEGIN TRANSACTION;
go
INSERT INTO TEMP_NATION
SELECT * FROM NATION
	WHERE N_NATIONKEY = 1;
go
UPDATE TEMP_NATION
	SET N_NATIONKEY = 2147483647;
go
INSERT INTO NATION
	SELECT * FROM TEMP_NATION;
go
SELECT * FROM NATION
	WHERE N_NATIONKEY = 2147483647
	OR N_NATIONKEY = 1;
go
DELETE FROM NATION
	WHERE N_NATIONKEY = 2147483647;
go
COMMIT;
go
BEGIN TRANSACTION;
go
INSERT INTO TEMP_REGION
SELECT * FROM REGION
	WHERE R_REGIONKEY = 1;
go
UPDATE TEMP_REGION
	SET R_REGIONKEY = 2147483647;
go
INSERT INTO REGION
	SELECT * FROM TEMP_REGION;
go
SELECT * FROM REGION
	WHERE R_REGIONKEY = 2147483647
	OR R_REGIONKEY = 1;
go
DELETE FROM REGION
	WHERE R_REGIONKEY = 2147483647;
go
COMMIT;
go
-- ==================================================================
-- DUPLICATES FINISHED STARTING INSERTS FOR DOMAIN RANGE
-- ==================================================================

-- GET TIMESTAMP
SELECT CONVERT(CHAR, GETDATE(), 120)
go

INSERT INTO SUPPLIER
	(S_SUPPKEY, S_NAME, S_ADDRESS, S_NATIONKEY, S_PHONE,
	S_ACCTBAL, S_COMMENT)
	VALUES
	(2147483647, 'NAME TEXT ............25E',
	'ADDRESS VARCHAR ............30.......40E',
	2147483647,'THIS IS PHONE E', 1234567890.12,
	'SUPPLIER COMMENT FIELD IS 101 LONG NO E');
go
SELECT * FROM SUPPLIER
	WHERE S_SUPPKEY = 2147483647;
go
DELETE FROM SUPPLIER
	WHERE S_SUPPKEY = 2147483647;
go
-- ==================================================================
INSERT INTO PART
	(P_PARTKEY, P_NAME, P_MFGR, P_BRAND, P_TYPE,
	P_SIZE, P_CONTAINER, P_RETAILPRICE, P_COMMENT)
	VALUES
	(2147483647, 'PNAME TEXT .......2.........3.........4....5E',
	'PMFGR TEXT........2....5E','PBRAND 10E',
	'PTYPE VARCHAR.....2....5E', -2147483646,
	'PCONTAINRE', 1234567890.12,
	'PART COMMENT FIELD  23E');
go
SELECT * FROM PART
	WHERE P_PARTKEY = 2147483647;
go
DELETE FROM PART
	WHERE P_PARTKEY = 2147483647;
go
-- ==================================================================
INSERT INTO PARTSUPP
	(PS_PARTKEY, PS_SUPPKEY, PS_AVAILQTY, PS_SUPPLYCOST,
	PS_COMMENT)
	VALUES
	(2147483647, 2147483647, -2147483646, 1234567890.12,
	'PS COMMENT FIELD IS 199 LONG NO E');
go
SELECT * FROM PARTSUPP
	WHERE PS_PARTKEY = 2147483647
	AND   PS_SUPPKEY = 2147483647;
go
DELETE FROM PARTSUPP
	WHERE PS_PARTKEY = 2147483647
	AND   PS_SUPPKEY = 2147483647;
go
-- ==================================================================
INSERT INTO CUSTOMER
	(C_CUSTKEY, C_NAME, C_ADDRESS, C_NATIONKEY,
	C_PHONE, C_ACCTBAL, C_MKTSEGMENT, C_COMMENT)
	VALUES
	(2147483647, 'CUSTOMER NAME GOES TO 25E',
	'CUSTOMER ADDRESS GOES HERE..3.........4E',
	2147483647, 'THIS IS PHONE E', 1234567890.12,
	'MARKT SEGE', 'CUSTOMER COMMENTS FIELS IS 117 LONG NO E');
go
SELECT * FROM CUSTOMER
	WHERE C_CUSTKEY = 2147483647;
go
DELETE FROM CUSTOMER
	WHERE C_CUSTKEY = 2147483647;
go
-- ==================================================================
INSERT INTO ORDERS
	(O_ORDERKEY, O_CUSTKEY, O_ORDERSTATUS, O_TOTALPRICE,
	O_ORDERDATE, O_ORDERPRIORITY, O_CLERK, O_SHIPPRIORITY,
	O_COMMENT)
	VALUES
	(2147483647, 2147483647, 'X', 1234567890.12,  getdate(),
	'ORDER PRIORTY5E', 'FIXED TEXT  15E', -2147483646,
	'ORDER COMMENTS FIELD IS 79 NO E');
go
SELECT * FROM ORDERS
	WHERE O_ORDERKEY = 2147483647
	AND  O_CUSTKEY = 2147483647;
go
DELETE FROM ORDERS
	WHERE O_ORDERKEY = 2147483647
	AND  O_CUSTKEY = 2147483647;
go
-- ==================================================================
INSERT INTO LINEITEM
	(L_ORDERKEY, L_PARTKEY, L_SUPPKEY, L_LINENUMBER,
	L_QUANTITY, L_EXTENDEDPRICE, L_DISCOUNT, L_TAX,
	L_RETURNFLAG, L_LINESTATUS, L_SHIPDATE, L_COMMITDATE,
	L_RECEIPTDATE, L_SHIPINSTRUCT, L_SHIPMODE, L_COMMENT)
	VALUES
	(2147483647,
	2147483647,
	2147483647,
	-2147483646,
	-1234567890.12,
	-1234567890.12,
	-1234567890.12,
	-1234567890.12,
	'Q',
	'R',
	getdate(),
	getdate(),
	getdate(),
	'SHIP BY CAMEL .........5E',
	'SHIP ASAPE',
	'IS THIS REALLY WHAT YOU WANTED? 44 LONG....E');
go
SELECT * FROM LINEITEM
	WHERE L_ORDERKEY = 2147483647
	AND L_PARTKEY = 2147483647
	AND L_SUPPKEY = 2147483647
	AND L_LINENUMBER = -2147483646;
go
DELETE FROM LINEITEM
	WHERE L_ORDERKEY = 2147483647
	AND L_PARTKEY = 2147483647
	AND L_SUPPKEY = 2147483647
	AND L_LINENUMBER = -2147483646;
go
-- ==================================================================
INSERT INTO NATION
	(N_NATIONKEY, N_NAME, N_REGIONKEY, N_COMMENT)
	VALUES
	(2147483647,
	'ZE REPUBLIC D MAKEBELIEVE',
	2147483647,
	'A NATION COMMENT FOR FIELD SIZE 152 NO E');
go
SELECT * FROM NATION
	WHERE N_NATIONKEY = 2147483647
	AND N_REGIONKEY = 2147483647;
go
DELETE FROM NATION
	WHERE N_NATIONKEY = 2147483647
	AND N_REGIONKEY = 2147483647;
go
-- ==================================================================
INSERT INTO REGION
	(R_REGIONKEY, R_NAME, R_COMMENT)
	VALUES
	(2147483647,
	'ZE ENDS OF THE EARTH....E',
	'A REASONABLE COMMENT WOULD GO HERE');
go
SELECT * FROM REGION
	WHERE R_REGIONKEY = 2147483647;
go
DELETE FROM REGION
	WHERE R_REGIONKEY = 2147483647;
go
-- GET TIMESTAMP
SELECT CONVERT(CHAR, GETDATE(), 120)
go
-- ==================================================================
-- DROP TEMP TABLES
-- ==================================================================
if exists (select name from sysobjects where name='TEMP_PART')
  drop table TEMP_PART
go
if exists (select name from sysobjects where name='TEMP_SUPPLIER')
  drop table TEMP_SUPPLIER
go
if exists (select name from sysobjects where name='TEMP_PARTSUPP')
  drop table TEMP_PARTSUPP
go
if exists (select name from sysobjects where name='TEMP_CUSTOMER')
  drop table TEMP_CUSTOMER
go
if exists (select name from sysobjects where name='TEMP_ORDERS')
  drop table TEMP_ORDERS
go
if exists (select name from sysobjects where name='TEMP_LINEITEM')
  drop table TEMP_LINEITEM
go
if exists (select name from sysobjects where name='TEMP_NATION')
  drop table TEMP_NATION
go
if exists (select name from sysobjects where name='TEMP_REGION')
 drop table TEMP_REGION
go
-- GET TIMESTAMP
SELECT CONVERT(CHAR, GETDATE(), 120)
go
-- ==================================================================
-- DONE
-- ==================================================================

