-- ==================================================================+
--  FILENAME
--       DBTABLES-SYB.SQL
--  DESCRIPTION
--       CHECK ROW COUNT AND ROW STRUCTURE/CONTENT FOR EACH TABLE
--       IN THE TPC-H DATABASE.
--
-- ==================================================================
--
-- GET TIMESTAMP
SELECT 'START TIME', CONVERT(CHAR(30), GETDATE(), 120);
go
-- ==================================================================
--           TABLE: LINEITEM
-- ==================================================================
SELECT COUNT(*) FROM LINEITEM;
go
SELECT l_orderkey, l_partkey, l_suppkey, l_linenumber, cast(l_quantity as NUMERIC(30,2)), cast(l_extendedprice as NUMERIC(30,2)), cast(l_discount as NUMERIC(30,2)), cast(l_tax as NUMERIC(30,2)), l_returnflag, l_linestatus, l_shipdate, l_commitdate, l_receiptdate, l_shipinstruct, l_shipmode, l_comment FROM LINEITEM
WHERE L_ORDERKEY IN
      ( 4, 26598,  148577,  3871431, 46556704, 1925174742, 6000000000)
      AND L_LINENUMBER = 1
ORDER BY L_ORDERKEY;
go
-- ==================================================================
--           TABLE: ORDERS
-- ==================================================================
-- GET TIMESTAMP
SELECT 'TIME', CONVERT(CHAR(30), GETDATE(), 120);
go
SELECT COUNT(*) FROM ORDERS;
go
SELECT o_orderkey, o_custkey, o_orderstatus, cast(o_totalprice as NUMERIC(30,2)), o_orderdate, o_orderpriority, o_clerk, o_shippriority, o_comment FROM ORDERS
WHERE O_ORDERKEY IN ( 7, 44065, 287590, 4111111, 24483876, 2901689942 ) 
ORDER BY O_ORDERKEY; 
go
-- ==================================================================
--           TABLE: PART
-- ==================================================================
-- GET TIMESTAMP
SELECT 'TIME', CONVERT(CHAR(30), GETDATE(), 120);
go
SELECT COUNT(*) FROM PART; 
go
SELECT p_partkey, p_name, p_mfgr, p_brand, p_type, p_size, p_container, cast(p_retailprice as NUMERIC(30,2)), p_comment FROM PART 
WHERE P_PARTKEY IN (1,9804,86743,976028,173876,16777899,500000000) 
ORDER BY P_PARTKEY; 
go
-- ==================================================================
--           TABLE: PARTSUPP
-- ==================================================================
-- GET TIMESTAMP
SELECT 'TIME', CONVERT(CHAR(30), GETDATE(), 120);
go
SELECT COUNT(*) FROM PARTSUPP;
go
SELECT ps_partkey, ps_suppkey, ps_availqty, cast(ps_supplycost as NUMERIC(30,2)), ps_comment FROM PARTSUPP
   WHERE PS_PARTKEY = 3398
   AND PS_SUPPKEY = (SELECT MIN(PS_SUPPKEY)
      FROM PARTSUPP WHERE PS_PARTKEY = 3398);
go
SELECT ps_partkey, ps_suppkey, ps_availqty, cast(ps_supplycost as NUMERIC(30,2)), ps_comment  FROM PARTSUPP
   WHERE PS_PARTKEY =1587553
   AND PS_SUPPKEY = (SELECT MIN(PS_SUPPKEY)
      FROM PARTSUPP WHERE PS_PARTKEY = 1587553);
go
SELECT ps_partkey, ps_suppkey, ps_availqty, cast(ps_supplycost as NUMERIC(30,2)), ps_comment  FROM PARTSUPP
   WHERE PS_PARTKEY = 1139412
   AND PS_SUPPKEY = (SELECT MIN(PS_SUPPKEY)
      FROM PARTSUPP WHERE PS_PARTKEY = 1139412);
go
SELECT ps_partkey, ps_suppkey, ps_availqty, cast(ps_supplycost as NUMERIC(30,2)), ps_comment  FROM PARTSUPP
   WHERE PS_PARTKEY = 67436543
   AND PS_SUPPKEY = (SELECT MIN(PS_SUPPKEY)
      FROM PARTSUPP WHERE PS_PARTKEY = 67436543);
go
SELECT ps_partkey, ps_suppkey, ps_availqty, cast(ps_supplycost as NUMERIC(30,2)), ps_comment FROM PARTSUPP
   WHERE PS_PARTKEY = 197637894
   AND PS_SUPPKEY = (SELECT MIN(PS_SUPPKEY)
      FROM PARTSUPP WHERE PS_PARTKEY =197637894);
go
-- ==================================================================
--           TABLE: SUPPLIER
-- ==================================================================
-- GET TIMESTAMP
SELECT 'TIME', CONVERT(CHAR(30), GETDATE(), 120);
go
SELECT COUNT(*) FROM SUPPLIER; 
go
SELECT s_suppkey, s_name, s_address, s_nationkey, s_phone, cast(s_acctbal as NUMERIC(30,2)), s_comment FROM SUPPLIER
WHERE  S_SUPPKEY IN (83,265,5492,78454,905871,3000000) 
ORDER BY S_SUPPKEY; 
go
-- ==================================================================
--           TABLE: CUSTOMER
-- ==================================================================
-- GET TIMESTAMP
SELECT 'TIME', CONVERT(CHAR(30), GETDATE(), 120);
go
SELECT COUNT(*) FROM CUSTOMER;
go
SELECT c_custkey, c_name, c_address, c_nationkey, c_phone, cast(c_acctbal as NUMERIC(30,2)), c_mktsegment, c_comment FROM CUSTOMER
WHERE  C_CUSTKEY IN (832,2653,4924,7845,92016,108070) 
ORDER BY C_CUSTKEY;
go
-- ==================================================================
--           TABLE: NATION & REGION
-- ==================================================================
-- GET TIMESTAMP
SELECT 'TIME', CONVERT(CHAR(30), GETDATE(), 120);
go
SELECT * FROM REGION;
go
SELECT COUNT(*) FROM NATION;
go
SELECT * FROM NATION
WHERE N_NATIONKEY IN (3,10,14,20) 
ORDER BY N_NATIONKEY; 
go
-- ==================================================================
--           CHECK KEY VALUES
-- ==================================================================
-- GET TIMESTAMP
SELECT 'TIME', CONVERT(CHAR(30), GETDATE(), 120);
go
if exists (select name from sysobjects where name='MINMAX')
  drop table MINMAX
go
CREATE TABLE MINMAX
(TNAME CHAR(15),
KEYMIN bigint,
KEYMAX bigint);
go
INSERT INTO MINMAX
SELECT 'LINEITEM_ORD',MIN(L_ORDERKEY),MAX(L_ORDERKEY)
FROM LINEITEM;
go
INSERT INTO MINMAX
SELECT 'LINEITEM_NBR',MIN(L_LINENUMBER),MAX(L_LINENUMBER)
FROM LINEITEM;
go
INSERT INTO MINMAX
SELECT 'ORDERS',MIN(O_ORDERKEY),MAX(O_ORDERKEY)
FROM ORDERS;
go
INSERT INTO MINMAX
SELECT 'CUSTOMER',MIN(C_CUSTKEY),MAX(C_CUSTKEY)
FROM CUSTOMER;
go
INSERT INTO MINMAX
SELECT 'PART',MIN(P_PARTKEY),MAX(P_PARTKEY)
FROM PART;
go
INSERT INTO MINMAX
SELECT 'SUPPLIER',MIN(S_SUPPKEY),MAX(S_SUPPKEY)
FROM SUPPLIER;
go
INSERT INTO MINMAX
SELECT 'PARTSUPP_PART',MIN(PS_PARTKEY),MAX(PS_PARTKEY)
FROM PARTSUPP;
go
INSERT INTO MINMAX
SELECT 'PARTSUPP_SUPP',MIN(PS_SUPPKEY),MAX(PS_SUPPKEY)
FROM PARTSUPP;
go
INSERT INTO MINMAX
SELECT 'NATION',MIN(N_NATIONKEY),MAX(N_NATIONKEY)
FROM NATION;
go
INSERT INTO MINMAX
SELECT 'REGION',MIN(R_REGIONKEY),MAX(R_REGIONKEY)
FROM REGION;
go
SELECT * FROM MINMAX;
go
if exists (select name from sysobjects where name='MINMAX')
  drop table MINMAX
go
SELECT 'END TIME', CONVERT(CHAR(30), GETDATE(), 120);
go
